package com.muyu.marketing.domain.model;


import com.fasterxml.jackson.annotation.JsonFormat;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.ActivityTeamOpenInfo;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.resp.ActivityTeamInfoResp;
import com.muyu.marketing.domain.resp.TeamProjectSkuInfoResp;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

/**
 * 团购活动查询模型
 *
 * @author DongZeLiang
 * @date 2024-11-26 09:38
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ActivityTeamInfoSelectModel {

    /**
     * 拼团活动id
     */
    private Long teamId;
    /**
     * 拼团名称
     */
    private String name;
    /**
     * 商品ID
     */
    private Long productId;

    /**
     * 商品活动图
     */
    private String productImage;
    /**
     * 活动简介
     */
    private String introduction;
    /**
     * 单位
     */

    private String unit;
    /**
     * 轮播图
     */

    private String imageList;
    /**
     * 活动结束时间
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd")
    private Date endTime;
    /**
     * 活动排序
     */
    private Long sort;
    /**
     * 商品SkU集合
     */
    private List<TeamProjectSkuInfoResp> teamProjectSkuInfoResp;
    /**
     * 活动详情
     */
    private String content;
    /**
     * 活动状态
     */
    private String status;
    /**
     * 策略类型
     */
    private String strategyType;
    /**
     * 策略ID
     */
    private Long strategyId;


    /** 回显 构造 ActivityTeamInfoSelectModel*/

    public static ActivityTeamInfoSelectModel getTeamInfoSelectModel(ActivityTeamInfo activityTeamInfo, List<ActivityTeamProductSkuInfo> list) {
        return ActivityTeamInfoSelectModel.builder()
                .teamId(activityTeamInfo.getId())
                .name(activityTeamInfo.getName())
                .productId(activityTeamInfo.getProductId())
                .productImage(activityTeamInfo.getProductImage())
                .introduction(activityTeamInfo.getIntroduction())
                .unit(activityTeamInfo.getUnit())
                .imageList(activityTeamInfo.getImageList())
                .endTime(activityTeamInfo.getEndTime())
                .sort(activityTeamInfo.getSort())
                .teamProjectSkuInfoResp(        list.stream().map(skuInfo ->
                        TeamProjectSkuInfoResp.getSkuInfoResp(skuInfo)
                ).toList())
                .content(activityTeamInfo.getContent())
                .status(activityTeamInfo.getStatus())
                .strategyType(activityTeamInfo.getStrategyType())
                .strategyId(activityTeamInfo.getStrategyId())
                .build();
    }
}
